Program empuxo1p1

! Marchi
! verso 1.1:  4 Jun 05
! verso 1.0: 30 Mai 05
! ltima alterao: 1 Nov 2013

USE PORTLIB

implicit none

real*8  :: z1, z2, z3
real*8  :: Vol, Rg, Tc, Ab, At, a, n, rop, gama, ac, cte, Vola, Vol0
real*8  :: L, t, dt, x, r, tol
real*8  :: Ri, W, pi, fmt, Ve, Fd, Cf, Fmed, Itot, Is, Ismed
real*8  :: p, pa, pao, paux, dp, Vtc, tq, z, g
real*8  :: Ai, Ae, Au, Voli, Vole, Volu, Re
integer :: passos, i, it, itmax, grao
integer :: ver, j

! dados

! grao:   1 = cilndrico (queima cigarro);
!         2 = tubular (queima interna);
!         3 = tubular (queima em toda superfcie)

grao =    3

gama =    1.156d0
Rg   =  213.7d0
Tc   = 1487.0d0
rop  = 1270.0d0
a    =    1.007d-4
n    =    0.319d0
!Vol0 =    3.664d-6     ! s para grao = 1
Vol0 =    2.392d-6    ! s para grao = 2 e 3
!Ab   =    3.664d-4     ! s para grao = 1
At   =    7.069d-6

pao  =  917.4d2
!L    =   10.0d-3     ! s para grao = 1 (comprimento do gro)
!L    =    7.005d-3    ! s para grao = 2 (espessura do gro)
L    =    3.5025d-3    ! s para grao = 3 (metade da espessura do gro)
W    =   17.00d-3     ! s para grao = 2 e 3 (comprimento do gro)
Ri   =    1.50d-3    ! s para grao = 2 e 3 (raio interno do gro)
Re   =    8.505d-3    ! s para grao = 3
Vtc  =    6.134d-6

dt     =     1.0d-4
passos = 80000 
x      =     0.0d0
itmax  =   100
tol    =     1.0d-5
pa     = pao

ac  = dsqrt(gama*Rg*Tc)
cte = gama*((2.0d0/(gama+1.0d0))**((gama+1.0d0)/(2.0d0*(gama-1.0d0))))
pi  = dacos(-1.0d0)
g   = 9.80665d0

open(8,file="saida.txt")

write(8,10)
10 format("#", t8,"i", t15,"tempo (s)", t30,"Fd (N)",   t45,"r (mm/s)", t60,"p (atm)", &
                  t75,"K (adim.)", t90,"fmt (kg/s)", t105,"Ve (m/s)",              &
                 t120,"x (mm)",   t135,"Cf (adim.)", t150,"Is (s)",   &
                 t165,"dp (Pa)",   t175,"iteraes")

! instante inicial

p   = pao
Vol = Vol0
r   = a*(p**n)
t   = 0.0d0
fmt = 0.0d0
Ve  = 0.0d0
Fd  = 0.0d0
Cf  = 0.0d0
Is  = 0.0d0
dp  = 0.0d0
Itot= 0.0d0
Fmed= 0.0d0 
it  = 0
i   = 0
if ( grao == 2 ) Ab = 2*pi*W*(Ri + x)

write(8,20) i, t, Fd, r*1000, p/101325.0d0, Ab/At, fmt, Ve, x*1000, Cf, Is, dp, it
20 format (i8, 11(1pe15.4), i8)

! combusto

select case (grao)

  case ( 1 )
    z1 = 1.0d0/(Rg*Tc)
    z2 = rop*Ab*a
    z3 = cte*At/ac

  case ( 2, 3 )
    z1 = 1.0d0/(Rg*Tc)
    z2 = rop*a
    z3 = cte*At/ac

end select

do i = 1, passos

   Vola = Vol

   p  = pa

   r  = a*(p**n)

   x  = x + r*dt

   t  = i * dt

   do it = 1, itmax

      paux = p

      select case (grao)

        case ( 1 )
          Vol = Vol0 + Ab*x
          p   = (Vola/Vol)*pa + ( z2*(p**n) - z3*p ) * dt / (Vol*z1) 

        case ( 2 )
          Ab  = 2*pi*W*(Ri + x)
          Vol = Vol0 + pi*W*( ((Ri+x)**2) - (Ri**2) )
          p   = (Vola/Vol)*pa + ( z2*Ab*(p**n) - z3*p ) * dt / (Vol*z1) 

        case ( 3 )
          Ai  = 2 * pi * (Ri+x) * (W-2*x)
          Ae  = 2 * pi * (Re-x) * (W-2*x)
          Au  = 2 * pi * ( ((Re-x)**2) - ((Ri+x)**2) )
          Ab  = Ai + Ae + Au
          Voli= pi * W * ( (Ri+x)**2 )
          Vole= pi * W * ( (Re**2) - ((Re-x)**2) )
          Volu= pi * 2 * x * ( ((Re-x)**2) - ((Ri+x)**2) )
          Vol = Vol0 + Voli + Vole + Volu
          p   = (Vola/Vol)*pa + ( z2*Ab*(p**n) - z3*p ) * dt / (Vol*z1) 

      end select

      dp = dabs(p-paux)

      if ( dp < tol ) exit

   end do

   fmt = cte*At*p/ac
   Ve  = ( 2*gama*Rg*Tc * (1.0d0 - ((pao/p)**((gama-1.0d0)/gama))) &
       / (gama-1.0d0) ) ** 0.5d0
   Fd  = fmt*Ve
   Itot= Itot + Fd*dt
   Is  = Fd/(fmt*g)
   Cf  = Fd/(p*At)
   Ismed = Ismed + Is*dt

   write(8,20) i, t, Fd, r*1000, p/101325.0d0, Ab/At, fmt, Ve, x*1000, Cf, Is, dp, it

   if ( x > L ) exit

   pa = p

end do

! despressurizao

z  = cte*At*Rg*Tc/(ac*Vtc)
pa = p
Ab = 0.0d0
r  = 0.0d0
dp = 0.0d0
it = 0
tq = t

do j = i+1, passos

   t  = j * dt

   p  = pa * dexp(-z*(t-tq))

   if ( p < pao ) exit

   fmt = cte*At*p/ac

   Ve  = ( 2*gama*Rg*Tc * (1.0d0 - ((pao/p)**((gama-1.0d0)/gama))) &
       / (gama-1.0d0) ) ** 0.5d0

   Fd  = fmt*Ve
   Itot= Itot + Fd*dt
   Is  = Fd/(fmt*g)
   Cf  = Fd/(p*At)
   Ismed = Ismed + Is*dt

   write(8,20) j, t, Fd, r*1000, p/101325.0d0, Ab/At, fmt, Ve, x*1000, Cf, Is, dp, it

end do

close(8)

Fmed  = Itot / t

Ismed = Ismed / t

open(8,file="saida2.txt")
write(8,30) tq, t-tq, t, Fmed, Ismed
30 format(/, 5x, "Tempo de queima (s)           = ", 1pe15.4, &
         //, 5x, "Tempo de despressurizao (s) = ", 1pe15.4, &
         //, 5x, "Tempo total com empuxo (s)    = ", 1pe15.4, &
         //, 5x, "Empuxo mdio (N)              = ", 1pe15.4, &
         //, 5x, "Impulso especfico mdio (s)  = ", 1pe15.4  )
close(8)

ver = system("Notepad saida.txt")

ver = system("Wgnuplot empuxo.gnu")

ver = system("Notepad saida2.txt")

end Program empuxo1p1
